#!/usr/bin/perl
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Name:    spcollect_backend.pl
# Revision: Is now tracked by the Base Revision
#
# Description: This is the script formerly known as spcollect.pl. It is designed to gather
# various
#
#
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#

###########################################################################################
#
# Revision History
#    ??-???-???? ??? Created from the original spcollect.pl
#    13-Nov-2006 MSZ Changed Revision to track to the Base Revision.
#                    Created new function:
#                    CopySpaceFile. Use to grab existing file that has space(s) in the path.
#    13-Mar-2007 MSZ DIMS 165821
#                    Add output of spstate to sus zip file.
#
###########################################################################################

use Getopt::Long;
use POSIX qw(strftime);
use Win32::EventLog;
use Win32::Service;
use Win32;
use File::Path;
use strict;

###########################################################################################
##########                     Set up some global values and constants           ##########
###########################################################################################

#Constants
use constant MAX_PREVIOUSSTATUS => 2;

#Predefined global values
my (
    $RETURN_GOOD,		#Global function return condition for success
    $RETURN_BAD,		#Global function return condition for failure
    $TRUE,			#Global true condition
    $FALSE,			#Global false condition
    $dumpdir,			#Directory where the collect zip will be saved
    $scriptname,		#The name presented to the system
    $revision,			#Current version presented to the system
    $tempdir,			#Directory where some known log files are held
    $minidumpdir,       #Directory where mini dumps are held
    $tempFileCount,    #Temporary File Counter

    # for use in putting headers into the output files generated by spcollect.
    $lineofstars,
    $lineofdashes
   );

$RETURN_GOOD = 0;
$RETURN_BAD = -1;
$TRUE = 1;
$FALSE = 0;
$dumpdir = "C:\\dumps";
$scriptname = "spcollect_backend.pl";
$revision = $ENV{'BASE_REVISION'};
$tempdir = "c:\\TEMP";
$minidumpdir = "C:\\WINNT\\Minidump";
$tempFileCount = 0;
$lineofstars = "********************************************************************************";
$lineofdashes = "--------------------------------------------------------------------------------";

#Global Variables
my (
    %zipHash,			#The files and their paths in the final zip
    %programHash,		#The programs and their dependencies used in the script
    %argHash,			#The arguments passed to the script
    %operatingHash,		#Commonly used paths and globals in the script
   );

###########################################################################################
##########                             prototypes                                ##########
###########################################################################################

# Copy existing file with spaces in the path, to one of the zips.
sub CopySpaceFile($$$$);

#copy an existing file to one of the zips
sub CopyFile($$$);

# Copy existing files to one of the zips
sub CopyFiles($$$$);

#called to create a log from the output of an executed program
sub CreateFile($$$$@);

#called to add a header label to a file being created
sub LabelLog($$@);

#called to send output stating the current status of the collect process
sub GiveStatus($@);

#called to verify whether a specific function call should run during restricted execution
sub CheckGroup($);

#check whether a layered app is needed and if so whether the required app is enabled
sub CheckLayeredApp($);

#check file size is > 0
sub CheckValidBinary($);

#executes a system command
sub InvokeCommand($$$);
sub InvokeCommand_($$$);

sub CaptureLccLogs($$);
sub CaptureLccStats($$);
sub CaptureNorthstarLogs($$);
sub CaptureHAVTLogs($$);
sub AddCreatedFileToZip($$);


###########################################################################################
##########                             subroutines                               ##########
###########################################################################################

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InitProgramHash
# usage : InitProgramHash()
#------------------------------------------------------------------------------------------
# This function assembles and organizes the list of programs that will be used
# throughout this script. The list of programs along with attributes necessary
# to run each program are placed inside a global hash.
#
# The general structure of the global hash is as follows:
# programHash{
#	"progShortName" => {
#		"name" => "progPath\progName", #path to the program
#		"services" => [service0, service1, etc..], #services needed for program
#		"previousStatus" => <val representing status> #completion state of last run
#		}
#	}
#------------------------------------------------------------------------------------------
sub InitProgramHash
  {
    #adding zip to the program hash
    $programHash{"zip"} = 
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\zip.exe",
       services => [],		#not dependent on services
       invokemode => "zip",
       previousStatus => 0	#good status by default
      };
	
    #adding navicli to the program hash
    $programHash{"navicli"} =
      {
       name => "c:\\EMC\\Navisphere\\$ENV{'NAVISPHERE_REVISION'}\\msgbin\\NaviCli.exe",
       services => ["flaredrv", "psm", "K10Governor"],
       invokemode => "UsesLayeredApp",
       previousStatus => 0	#good status by default
      };

    #adding naviseccli to the program hash
    $programHash{"naviseccli"} =
      {
       name => "c:\\EMC\\Navisphere\\$ENV{'NAVISPHERE_REVISION'}\\msgbin\\NavisecCli.exe",
       services => ["flaredrv", "psm", "K10Governor"],
       invokemode => "UsesLayeredApp",
       previousStatus => 0	#good status by default
      };

    #adding navicimom to the program hash
    # NOTE navicimcli is naviseccli.
    # All references to navicimom need to be changed to naviseccli
    # and the CAP parsing code update to search for both naviseccli an navicimcli
    $programHash{"navicimom"} =
      {
       name => "c:\\EMC\\ManagementServer\\$ENV{'MANAGEMENTSERVER_REVISION'}\\CIMOM\\NaviCIMCli.exe",
       services => ["flaredrv", "psm", "K10Governor"],
       invokemode => "UsesLayeredApp",
       previousStatus => 0	#good status by default
      };

    #adding mvacli to the program hash
    $programHash{"mvacli"} =
      {
       name => "c:\\EMC\\MVAEngine\\$ENV{'MVAEngine_REVISION'}\\msgbin\\mvacli.exe",
       services => ["flaredrv", "psm", "K10Governor"],	
       invokemode => "UsesLayeredApp",
       previousStatus => 0	#good status by default
      };
 
    #adding hostconfcli to the program hash
    $programHash{"hostconfcli"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\flare_driver\\bin\\hostconfcli.exe",
       services => ["flaredrv"],
       invokemode => "normal",
       previousStatus => 0	#good status by default
      };

    #adding ktcons to the program hash
    $programHash{"ktcons"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\ktcons.exe",
       services => ["flaredrv"],	
       invokemode => "ktcons",
       previousStatus => 0	#good status by default
      };


    #adding psmtool to the program hash
    $programHash{"psmtool"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\psmtool.exe",
       services => ["flaredrv", "psm"],
       invokemode => "normal",
       previousStatus => 0	#good status by default
      };

    #adding getspids to the program hash
    $programHash{"getspids"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\getSpIds.exe",
       services => ["flaredrv"],	
       invokemode => "normal",
       previousStatus => 0	#good status by default
      };


    #adding route to the program hash
    $programHash{"route"} =
      {
       name => "$ENV{'SystemRoot'}\\system32\\route.exe",
       services => [],	
       invokemode => "normal",
       previousStatus => 0	#good status by default
      };

    #adding ipconfig to the program hash
    $programHash{"ipconfig"} =
      {
       name => "$ENV{'SystemRoot'}\\system32\\ipconfig.exe",
       services => [],	
       invokemode => "normal",
       previousStatus => 0	#good status by default
      };

    #adding netsh to the program hash
    $programHash{"netsh"} =
      {
       name => "$ENV{'SystemRoot'}\\system32\\netsh.exe",
       services => [],	
       invokemode => "normal",
       previousStatus => 0	#good status by default
      };

    #adding set_nic_config.pl to the program hash
    $programHash{"set_nic_config.pl"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\set_NIC_config.pl",
       services => [],	
       invokemode => "normal",
       previousStatus => 0	#good status by default
      };

    #adding SPCYukonCollect utility
    $programHash{"spcyukoncollect"} =
      {
        name => "C:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\spcyukoncollect.exe",
        services => ["flaredrv"],
        invokemode => "normal",
        previousStatus => 0
      };

    #adding MonitorFBI to program hash
    $programHash{"monitorfbi"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\MonitorFBI.exe",
       services => "",
       invokemode => "normal",
       previousStatus => 0
      };

    #adding AdminTool to program hash
    $programHash{"admintool"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\AdminTool.exe",
       services => [],
       invokemode => "normal",
       previousStatus => 0
      };

    $programHash{"smbtool"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\smbtool.exe",
       services => [],
       invokemode => "normal",
       previousStatus => 0
      };

    #adding spstate.exe to program hash
    $programHash{"spstate"} =
      {
       name => "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin\\spstate.exe",
       services => [],
       invokemode => "normal",
       previousStatus => 0
      };

  }				#end InitProgramHash

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InitValues
# description: This funciton will gather several pieces of SP and run specific
# data that will be used throughout the script. These values will be assigned 
# to a global hash.
#------------------------------------------------------------------------------------------
sub InitValues
  {
    #Local Variables
    my($cmd_output, $status);

    #System information that will be used throughout the script
    $operatingHash{base_msgbin_dir} = "c:\\EMC\\Base\\$ENV{'BASE_REVISION'}\\msgbin";
    $operatingHash{mirrorview_msgbin_dir} = "c:\\EMC\\MirrorView\\$ENV{'MIRRORVIEW_REVISION'}\\msgbin";
    $operatingHash{ini_file_path} = $operatingHash{base_msgbin_dir} . "\\spcollect.ini";
    $operatingHash{spcollect_path} = $operatingHash{base_msgbin_dir} . "\\spcollect_backend.pl";

    #Get SP identification
    $cmd_output = `query_sp_id`;
    $status = $?/256;
    if ( $status == 0 ) {
      $operatingHash{sp_id} = "SPA";
    } elsif ( $status == 1 ) {
      $operatingHash{sp_id} = "SPB";
    } else {
      $operatingHash{sp_id} = "SPX";
    }

    #Setup the Hash that holds the list of programs to be used and requirements
    &InitProgramHash;

    #Set the timestamp that will be used throughout the script
    $operatingHash{file_date} = &FilenameDT();

    ############################################################################
    ##### Grab SP specific info if navicli is available ########################
    ############################################################################

    if (&CanExecute("navicli")) {

      ## Get the SP's Serialnumber ##
      my $line = `$programHash{navicli}{name} getagent -serial`;
      chomp($line);

      #Make sure that the serialnumber was returned and not an error
      if ( $line =~m/\s*Serial\s*No.\s*(\S*)/ ) {
	$operatingHash{sp_serial} = $1;
      } else {
	$operatingHash{sp_serial} = "NOSERIALNUM";
      }

      ## Get the SP's Signature ##
      $line = `$programHash{navicli}{name} getagent -sig`;
      chomp($line);

      #Make sure that the signature was returned and not an error
      if ( $line =~m/\s*Signature.\s*(\S*)/ ) {
	my $hexsig = sprintf("%X", $1);
  
	$operatingHash{sp_sig} = lc($hexsig);
	} else {
	$operatingHash{sp_sig} = "NOSIG";
      }
    } else {
      $operatingHash{sp_serial} = "NOSERIALNUM";
      $operatingHash{sp_sig}    = "NOSIG";
    }

    ##The naming of spcollect impacts  
    ##1. PeerWatch library: check PeerWatchDefs.h file 
    ##2. CAP: Check ZipProcess.java file 
    
    ## Define Identifiers, used for naming zip's, files, and header messages throughout the script ##
    $operatingHash{zip_identifier} = $operatingHash{sp_serial} . "_" . $operatingHash{sp_id} . "_" .
      $operatingHash{file_date}  . "_" . $operatingHash{sp_sig} . "_";

    $operatingHash{file_name_identifier} = $operatingHash{sp_serial}. "_" .  $operatingHash{sp_id} . "_" .
      $operatingHash{file_date} . "_" . $operatingHash{sp_sig} . "_";

    $operatingHash{log_identifier} = "#spcollect " . $operatingHash{sp_serial} . "_" . $operatingHash{sp_id} . "_" . 
       $operatingHash{file_date} . "_" . $operatingHash{sp_sig};

    ## Define entries for each zip file in the zipHash ##
    $zipHash{"sus"} = {name => $operatingHash{zip_identifier} . "sus.zip"};
    $zipHash{"isc"} = {name => $operatingHash{zip_identifier} . "isc.zip"};
    $zipHash{"fbi"} = {name => $operatingHash{zip_identifier} . "fbi.zip"};
    $zipHash{"rtp"} = {name => $operatingHash{zip_identifier} . "rtp.zip"};
    $zipHash{"nav"} = {name => $operatingHash{zip_identifier} . "nav.zip"};
    $zipHash{"evt"} = {name => $operatingHash{zip_identifier} . "evt.zip"};
    $zipHash{"ktd"} = {name => $operatingHash{zip_identifier} . "ktd.zip"};
    $zipHash{"psm"} = {name => $operatingHash{zip_identifier} . "psm.zip"};
    if (CheckGroup ("")){#don't gather  minidump in brief mode
    	$zipHash{"minidump"}  = {name => $operatingHash{zip_identifier} . "minidump.zip"}; 
    	$zipHash{"lcc"}       = {name => $operatingHash{zip_identifier} . "lcc.zip"}; 
    	$zipHash{"drive"}     = {name => $operatingHash{zip_identifier} . "drive.zip"}; 
    	$zipHash{"dumpanalysis"}       = {name => $operatingHash{zip_identifier} . "dumpanalysis.zip"}; 
    }
  }				#end InitValues

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : AddCreatedFileToZip
# description: update the appropriate zip hash to include a reference to a created file
# function will display the correct command line usage framework to STDOUT
#------------------------------------------------------------------------------------------
sub AddCreatedFileToZip($$)
  {
    my ($zip, $fileName) = @_;
    $zipHash{$zip}{$fileName} = "created";
  }


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : Usage
# description: in the event that invalid command line values are passed to spcollect this
# function will display the correct command line usage framework to STDOUT
#------------------------------------------------------------------------------------------
sub Usage
  {
    #Local Variable
    my ($switchlist);

    #display the necessary usage information to the user
    $switchlist = "[-help] [-mode brief]";
    print "Usage:\n";
    print "   spcollect.pl $switchlist\n";
    print "    -help       : Display this message\n";
    print "    -mode brief : when specified, selects subset to be captured\n";
    print "                  brief defines smallest/fastest collect mode\n";
    exit;
  }				#end Usage

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetGMTime
#------------------------------------------------------------------------------------------
# Returns a hash containing time and date from gmtime
# function.
#------------------------------------------------------------------------------------------
sub GetGMTime
  {
    my %gmtimeHash;
    @gmtimeHash{"sec", "min", "hour", "day", "mon", "year", "wday", "yday"} = gmtime;

    # Adjust the year value
    $gmtimeHash{year} += 1900;

    # Adjust the month value
    $gmtimeHash{mon}++;

    return %gmtimeHash;
  }       # end GetGMTime

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : FormatTimeHash
# parameters : $timehash - Reference to hash created by either localtime or gmtime
#------------------------------------------------------------------------------------------
# Adjusts the contents of localtime or gmtime hash with
# leading zeros for digits below 10.
#------------------------------------------------------------------------------------------
sub FormatTimeHash($)
  {
    my ($timehash, $item);
    $timehash = $_[0];
    # adjust other single digits to have a leading zero
    foreach $item (keys %{$timehash}) {
      if (${$timehash}{$item} < 10) {
        ${$timehash}{$item} = "0" . ${$timehash}{$item};
      }
    }
  }
  
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : FilenameDT
#------------------------------------------------------------------------------------------
# Use the GetGMTime function to get a time and date
# stamp for the construction of the output filename.
#------------------------------------------------------------------------------------------
sub FilenameDT
  {
    #Local Variables
    my(%timehash);

    # get the time
    %timehash = GetGMTime();

   # adjust other single digits to have a leading zero
    FormatTimeHash(\%timehash);

    #Construct the final time string to return
    my $filedate =  $timehash{year}. "-" . $timehash{mon} . "-" . $timehash{day} . "_" .
      $timehash{hour} . "-" . $timehash{min} . "-" . $timehash{sec};

    return $filedate;
  }				#end FilenameDT

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : RedirectSTDOUT
#------------------------------------------------------------------------------------------
# Redirects STDOUT to a temporary file.
#------------------------------------------------------------------------------------------
sub RedirectSTDOUT($)
  {
    # Temp file name
    my ($tempFile) = @_;

    #Save current STDOUT and redirect
    open (OLDSTDOUT, ">&STDOUT") or die "Attempting to redirect STDOUT, failed to save current STDOUT: $!";
    open (STDOUT, "> $tempFile") or die "Failed to redirect STDOUT to $tempFile: $!";
  }        #end RedirectSTDOUT

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : RestoreSTDOUT
#------------------------------------------------------------------------------------------
# Restores previous STDOUT stream.
#------------------------------------------------------------------------------------------
sub RestoreSTDOUT
  {
    #Close the redirected stream and reconnect
    close STDOUT;
    open (STDOUT, ">& OLDSTDOUT") or die "Can't restore STDOUT: $!";

    close OLDSTDOUT;
  }        #end RestoreSTDOUT

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CopyFileTo
# parameters : $originalFile - Path to original file
#              $targetFile   - Path to target file to append to
#------------------------------------------------------------------------------------------
# Appends the contents of the file specified by the first argument to
# the file specified by the second argument.  Creates the second file
# if it does not exist.
#------------------------------------------------------------------------------------------
# bcm - 01/28/06 Created
#------------------------------------------------------------------------------------------
sub CopyFileInto($$)
  {
    my($originalFile, $targetFile) = @_;

    #Create handles to files
    open (ORGFILE, "< $originalFile")
      or print STDERR "CopyFileInto Could not open original file ($originalFile): $!\n";
    open (TGTFILE, ">> $targetFile") 
      or print STDERR "CopyFileInto Could not open target file ($targetFile): $!\n";

    while (<ORGFILE>) {
      print TGTFILE;
    }

    #Close handles
    close TGTFILE;
    close ORGFILE;
  }        #end CopyFileInto

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetTempFilename
#------------------------------------------------------------------------------------------
# Returns a unique temporary file path in the working directory
#------------------------------------------------------------------------------------------
# bcm - 01/28/06 Created
#------------------------------------------------------------------------------------------
sub GetTempFilename()
  {
    my $tempFile;
    $tempFile = $operatingHash{working_dir}."\\TEMP_";
    $tempFile = $tempFile.sprintf("%03d",$tempFileCount++);

    return $tempFile;
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CleanupIncompleteRuns
#------------------------------------------------------------------------------------------
# In the event that a previous spcollect process exited before completing
# successfully, we must do some housekeeping. We find all previously
# created temporary working direcotories, zip up the files in those directories
# and then delete the working directories.
#------------------------------------------------------------------------------------------
sub CleanupIncompleteRuns
  {
    #Local Variabls
    my (@leftOverDirs, $dumpdirContent, $oldCollectFile,
  	$zipOutput, $counter, $oldDir, $dirsString);
  	
  	my (@leftOverRunlogs, %runloghash, $dir, $ranTime,$logName, $fulloldLogPath, $destLog ); 

    #Get a listing of all the files and directories in the dumps directory
    $dumpdirContent = `dir $dumpdir`;

    #Extract the old working directory names from the directory listing
    @leftOverDirs = $dumpdirContent =~ /<DIR>\s*(spcollect_\S+)\n/g;

        
    #Extract the old runlog names from the directory listing
    @leftOverRunlogs = $dumpdirContent =~ /\s*(\S+runlog\S+)\s*\n/g;
        
    #construct a HASHMAP of spcollectdir and runlog name as key&value. 
    %runloghash = ();
	foreach $dir (@leftOverDirs) {
		if($dir =~ /(\d{4}\D+\d{2}\D+\d{2}\D+\d{2}\D+\d{2}\D+\d{2})/){
			$ranTime  = $1;  
			foreach $logName  (@leftOverRunlogs){
				if( $logName =~ /$ranTime/){	
					$runloghash { $dir } = $logName; 
				}
			}
		}
	}
	
    ##Append the full path to each old working directory##
    $counter = 0;
    foreach $oldDir (@leftOverDirs) {
	  #move the left over runlog to the $oldDir. 
	  $logName = $runloghash{ $oldDir }; 
	  if($logName){
		$fulloldLogPath = $dumpdir."\\".$logName;
		$destLog = $dumpdir . "\\". $oldDir."\\".$logName;

		CreateCopyOfFile($fulloldLogPath, $destLog); 
	  }	  
	    
	  $leftOverDirs[$counter] = $dumpdir . "\\" . $oldDir;
      $counter++;
    }

    #delete all old runlogs 
	foreach $logName  (@leftOverRunlogs){
		$fulloldLogPath = $dumpdir."\\".$logName;		
		unlink $fulloldLogPath; 
	}   
    
    ##Create a zip file to hold any leftover information
    if (@leftOverDirs) {
      #Create a name for the collection of leftover directories
      $oldCollectFile = $dumpdir . "\\" . &GenUniqueLogName("old_data.zip");

      #Put all of the leftover directory paths into a single string
      $dirsString = join(" ", @leftOverDirs);

      #Put all old working directories and delete them from the system
      $zipOutput = `$programHash{zip}{name} -r -m $oldCollectFile $dirsString`;
    }
  }				#end CleanupIncompleteRuns

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InitRuntimeFiles
#------------------------------------------------------------------------------------------
# Create spcollect zip filename along with the run specific log names
#------------------------------------------------------------------------------------------
sub InitRuntimeFiles()
  {
    #Create the directory that will be used to store temporary files
    $operatingHash{working_dir} = $dumpdir . "\\" . "spcollect_" . $operatingHash{file_date};
    mkpath($operatingHash{working_dir});

    #Set the working directory as the current directory
    chdir($operatingHash{working_dir});

    # Create the name of the SPCollect zip file:
    $zipHash{"name"} = $operatingHash{zip_identifier} . "data.zip";

    #Output the name of the Collect file
    print "\n\nThe SP collect filename will be\n";
    print "    $zipHash{name}\n\n";

    ###########################################################################
    ##We also need to create a log file to record what we do, stderr, and    ##
    ##stdout.								     ##
    ###########################################################################
    ## The following files are runtime log files that will be modified       ##
    ## throughout the script. They will not go onto the final list of files  ##
    ## to be zipped until they have been completed and closed                ##

    #Add entries for the log stdout, stderr, and "log" file into the zip Hash
    $operatingHash{log_file} = $dumpdir . "\\".&GenUniqueLogName("runlog.txt");
    
    $operatingHash{stdo_file} = &GenUniqueLogName("stdout_log.txt");
    $operatingHash{stde_file} = &GenUniqueLogName("stderr_log.txt");

  }				#end InitRuntimeFiles

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : RestoreStd
#------------------------------------------------------------------------------------------
# This function takes the redirected STDERR stream and puts it back to its normal 
# orientation.
#------------------------------------------------------------------------------------------
# wmb - 5/19/03 - Created
# dad - 11/04 - STDOUT removed from this function
#------------------------------------------------------------------------------------------
sub RestoreStd
  {
    #Close the redirected stream
    close STDERR;
	
    #Reconnect the stream
    open (STDERR, ">&OLDSTDERR") or die "Can't restore STDERR: $!";

    #Close the old file handle
    close OLDSTDERR;
  }				#end RestoreStd

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : RedirectStd
#------------------------------------------------------------------------------------------
# This function redirects the STDERR stream.
#------------------------------------------------------------------------------------------
# wmb - 5/19/03 - Created
# dad - 11/04 - STDOUT removed from this function
#------------------------------------------------------------------------------------------
sub RedirectStd
  {
    #First, duplicate the old STDERR stream so that we can restore it later
    open (OLDSTDERR, ">&STDERR") or die "Couldn't dup STDERR: $!";

    #Now redirect the stream

    open (STDERR, ">> $operatingHash{stde_file}") or die "Can't open $operatingHash{stde_file}: $!";

    #Make the streams unbuffered
    select(STDERR); $| = 1;
  }				#end RedirectStd

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : LogAction
#------------------------------------------------------------------------------------------
# Make a log entry that we are kicking off an spcollect.
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Put all of this code together in its own sub
#------------------------------------------------------------------------------------------
sub LogAction
  {
    # set up the logging executable
    my $dir = "\\EMC\\Base\\$ENV{ 'BASE_REVISION' }";

    if ( -f "$dir\\errgenlog.exe" ) {
      # Make a log entry... do it here so that the logs we gather later will have this in them.
      `$dir\\errgenlog.exe log_msg info "spcollect: Rev: $revision, file: $zipHash{name}"`;
      `$dir\\errgenlog.exe trace_msg info "spcollect: Rev: $revision, file: $zipHash{name}"`;
    }

  }				#end LogAction

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : LogSystemTime
#------------------------------------------------------------------------------------------
# Constructs a string displaying the GMT time and the local
# time reported by Perl's gmtime and localtime respectively.
#------------------------------------------------------------------------------------------
# bcm - 01/20/06 - created
#------------------------------------------------------------------------------------------
sub LogSystemTime
  {
    #local variables
    my(%localhash,$localTimeZone, $timeDif,%gmthash,$timeString,$item,$DST);

    # Get GMT Time
    %gmthash = GetGMTime();
    # Get Local Time
    @localhash{"sec", "min", "hour", "day", "mon", "year", "wday", "yday", "isdst"} = localtime;

    #Format time hashes
    FormatTimeHash(\%gmthash);
    $localhash{mon}++;
    $localhash{year} += 1900;
    if($localhash{isdst} == 1) {
      $localhash{hour}--;
      $DST = " DST"
    }
    FormatTimeHash(\%localhash);

    # Get timezone using POSIX strftime function
    $localTimeZone = strftime("%Z",localtime);
    $timeDif = $localhash{hour} - $gmthash{hour};

    # Create GMT/Local Time String
    $timeString = "System Time: $gmthash{year}-$gmthash{mon}-$gmthash{day} " . 
    "$gmthash{hour}:$gmthash{min}:$gmthash{sec} GMT" .
    "    $localhash{year}-$localhash{mon}-$localhash{day} " .
    "$localhash{hour}:$localhash{min}:$localhash{sec} (GMT $timeDif:00) $localTimeZone $DST";

    RunLogIt($timeString);
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherVersionInfo
#------------------------------------------------------------------------------------------
# Figure out which version and dumpdir we are using, and put that in our records.
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Put all of this code together in its own sub
#------------------------------------------------------------------------------------------
sub GatherVersionInfo
  {
    #local variable
    my($file_name);

    #create a unique name for the log
    $file_name = &GenLogName("collect_name_rev.txt");
	
    #Create an entry for the version info file in the zip Hash
    $operatingHash{fname_name_rev} = $file_name;

    #Add header and timestamp to the version info file
    &HiliteLogMessage($operatingHash{fname_name_rev}, 
		      "Name of collect script: $scriptname",
		      "Version of collect script: $revision");
    &LogMessage($operatingHash{fname_name_rev}, 
		"Start time of operation: " . $operatingHash{file_date});
  }				#end GatherVersionInfo

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : PreRunSetup
#------------------------------------------------------------------------------------------
#The purpose of this function is to allow us to check a number of conditions
#at/near the beginning of run-time. We also initialize many values and files used later.
#------------------------------------------------------------------------------------------
#wmb - Created 3/03
#------------------------------------------------------------------------------------------
sub PreRunSetup
  {
    #test for valid command line arguments 
    unless (GetOptions(\%argHash, 'help', 'mode=s'))

      {
	&Usage();
      }

    #provide help on usage if requested
    &Usage() if($argHash{'help'} || $argHash{'h'});

    #setup global values that will be used throughout the script
    &InitValues();

    #check for old spcollect working directories
    &CleanupIncompleteRuns();

    #Setup some of the files and and the working directory
    &InitRuntimeFiles();

    #Create top level log containing version and timestamp info
    &GatherVersionInfo();

    #Send a "note" to the log file saying that we are being invoked
    &LogAction();

    #Pipe STDERR to other streams
    &RedirectStd();

    #Note GMT and Local Time in run log
    LogSystemTime();

    #Log Run mode type
    my $mode;
    if (defined $argHash{'mode'}) {
      $mode = $argHash{'mode'};
    } else {
      $mode = "Full";
    }
    &RunLogIt("SPCollect Mode: $mode");

    #pause the fbi service now, so we can grab the files later
    &InvokeCommand($programHash{"monitorfbi"}{name}, "-pause", "");

  }				#end PreRunSetup

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetNaviMngrServerData
#------------------------------------------------------------------------------------------
# Purpose: Add cimom hang info files to the "nav" zip
#------------------------------------------------------------------------------------------
# kcd - 09/04 - Added lastNTEvtIndices.log to list
# dad - 11/04 - Stripped navi files out of function and put them into ini file
#------------------------------------------------------------------------------------------
sub GetNaviMngrServerData()
  {
    #Local Variables
    my (@cimomFiles);

    #outputing to the stdout log
    &GiveStatus("", "Status: Gather cimom files from spcollect");

    #check for CIMOM_hanginfo files
    opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
    @cimomFiles = grep /^CIMOM_hanginfo.*\.txt$/i, readdir DUMP;
    closedir DUMP;

    #add the files to the list of files to be zipped
    &SaveFilesToZip("nav", @cimomFiles);	

  }				#end GetNaviMngrServerData

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetNtEventLogs
#------------------------------------------------------------------------------------------
# This function gathers all the NT Event logs together.
#------------------------------------------------------------------------------------------
sub GetNtEventLogs
  {
    #Local Variables
    my ($myServer, $eventLog, $handle, $dest, $logFile);

    #Output that the script is now getting NT event logs
    &GiveStatus("", "Status: Gather NT Event Logs");

    #Get the name of the SP from the environment variables
    $myServer="\\\\$ENV{ComputerName}";

    ##Walk through each event log and save the log to a file that will be    ##
    ##added to the evt zip						     ##
    foreach $eventLog ("Application", "System", "Security") {
      #Create a new event log object from which to extract the log data
      $handle=Win32::EventLog->new($eventLog, $myServer)
	or die "Can't open $eventLog EventLog on $myServer\n";

      #Create a file to hold the event log data
      $logFile = &GenLogName($eventLog . ".evt");

      #Save a backup of the event log data to the file
      $handle->Backup($logFile)
	or warn "Could not backup the $eventLog EventLog on $myServer ($^E)\n";

      #add the file with the event log data to the list of files to be zipped
      &AddCreatedFileToZip("evt", $logFile);
    }

    # Close must be called last!!
    #
    # for unknown reasons, this above code fails when Close is called
    # within the loop.  The first pass completes, the rest fail..
    # I suspect that the problem is with the old version of perl used
    # on the systems.
    #
    $handle->Close;

  }				#end GetNtEventLogs

  
   
  
  
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetMiniDumps
#-----------------GetMiniDumps------------------------------------------------------------------------
# This function gathers all the minidumps under the dir "C:\WINNT\Minidump". 
#------------------------------------------------------------------------------------------
sub GetMiniDumps
  {
    #Local Variables
    my ( $file, @minidumpfiles);

    if (CheckGroup ("")){#don't gather  minidump in brief mode
	    #Output that the script is now getting minidumps
	    &GiveStatus("", "Status: Gather minidumps");
	
	       
	    my $openDirResult = opendir MINIDUMPDIR, "$minidumpdir";
		if($openDirResult){
	        @minidumpfiles = grep{/Mini.*\.dmp/i}readdir MINIDUMPDIR;
		    closedir MINIDUMPDIR;
		
		    
		    #Traverse the list of files appending the complete path to them
		    #Add each file to the appropriate list of files to be zipped
		    foreach $file (@minidumpfiles) {
		      # Add each of the .dmp files to the list of files to be zipped
		      SaveFilesToZip("minidump",$minidumpdir . "\\" . $file);
		    }
		}else{
	    	&GiveStatus("", "Status: " . $minidumpdir . " not found. Can't gather mini dumps.");
	    	print STDERR $minidumpdir . " not found.\n" ;		
		}
    }
  }	#end GetMiniDumps

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GetAnalysisFiles
#-----------------GetAnalysisFiles------------------------------------------------------------------------
# This function gathers all the analysis files under the dir "C:\Dumps". 
#------------------------------------------------------------------------------------------
# mwh - 7/05 - Created
#------------------------------------------------------------------------------------------
sub GetAnalysisFiles
  {
    #Local Variables
    my ( $file, @analysisfiles);

    if (CheckGroup ("")){#don't gather  minidump in brief mode
        #Output that the script is now getting analysis files
        &GiveStatus("", "Status: Gather dump analysis files");
            
        my $openDirResult = opendir DUMPDIR, "$dumpdir";
        if($openDirResult){
            @analysisfiles = grep {/^SP[\w\W]*.analysis.txt$/} readdir DUMPDIR;
            closedir DUMPDIR;
            
            #Traverse the list of files appending the complete path to them
            #Add each file to the appropriate list of files to be zipped
            foreach $file (@analysisfiles) {
                # Add each of the .dmp files to the list of files to be zipped
	        SaveFilesToZip("dumpanalysis", $dumpdir . "\\" . $file);
            }
        }else{
            &GiveStatus("", "Status: " . $dumpdir . " not found. Can't gather dump analysis files.");
            print STDERR $dumpdir . " not found.\n" ;       
        }
     }
  }	#end GetAnalysisFiles


  
  
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub - GetNaviGetLog
#------------------------------------------------------------------------------------------
# This function looks at the ktcons log for propogation of the message
# that we sent notifying ktcons about our running. If it is not there by the time that we
# check then we save the log off and insert our own message to the effect. The log is also
# added to the appropriate list of files to be zipped
#------------------------------------------------------------------------------------------
sub GetNaviGetLog
  {
    #Local Variables
    my (%timehash,  @grep_string, @output);
    my ($navilog_exec, $i, $logFile, $logForEdit);

    # Output that the script is getting SP Event Log
    &GiveStatus("brief", "Status: Gather the SP Event Log via navicli getlog from spcollect.");
    

    #Capture the log and add it to the appropriate list of files to zip
    &CreateFile("", "sus", "navi_getlog.txt", "navicli", "getlog");

    #Create the filename of the log to edit
    $logFile = &GenLogName("navi_getlog.txt");

    #open the log file just captured to check its contents
    open(NAVI_LOG, "<$logFile");

    #if we don't find the collect file name in the log then enter it manually
    if (!(<NAVI_LOG> =~ m/$zipHash{name}/i)) {
      #close the log file so that it can be opened it for editing
      close(NAVI_LOG);
	
      #open the log file so that it can be appended to
      open(NAVI_LOG, ">>$logFile");

      # get the time
      %timehash = GetGMTime();

      # adjust other single digits to have a leading zero
      FormatTimeHash(\%timehash);

      #Construct the final time string to add
      my $filedate = $timehash{mon} . "/" . $timehash{day} . "/" . $timehash{year} . " " .
	$timehash{hour} . ":" . $timehash{min} . ":" . $timehash{sec};

      #add the missing entry to the end of the file
      printf(NAVI_LOG "$filedate\t\t (71510000)Informational Message. " . 
	     "spcollect: Rev: $revision, file: $zipHash{name}\n 71 51 00 00\t\t\t\tNDU\n");

      #close the log
      close(NAVI_LOG);
    }
  }				#end GetNaviGetLog

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub - GatherPromInfo
#------------------------------------------------------------------------------------------
# Use ktcons to collect prom info
#------------------------------------------------------------------------------------------
# Uses ktcons !getprominfo commands.  Ultimately, a file called prom_info.txt will be
# placed in the list of files for sus.zip file.  Produces one ktcons input file per bus
# to get around a ktcons limitation on the number of commands that can be put into an
# input file.  Each enclosure on the bus produces a separate ktcons log file so we can
# put a header on it.  Otherwise, it is hard to visually separate the components.
#------------------------------------------------------------------------------------------
sub GatherPromInfo
  {
    my ($BundleIndexRevision, $hw_type, @buses, $bus, $starting_enclosure, $ktini,
	$ktini_path, $enclosure_id, $prom_info_file, $prom_info_path, $prom_id,
	$first_ktcons_ini_path, $bus_enclosure, @ktcons_log_files, $prom_path, $file,
	$ZipIdentifier);

    # See what type of platform we are.  If the environment variable BUNDLEINDEX_REVISION
    # does not exist, then the hardware type will be set to "none".

    $BundleIndexRevision = $ENV{'BUNDLEINDEX_REVISION'};

    # The BUNDLEINDEX_REVISION has the form mmnnhhhx.yyy, where hhh is the platform.
    # For example, 02114003.008; we want to extract the 400 out of this string.

    if ($BundleIndexRevision =~ /(\d\d)(\d\d)(\d\d\d)/) {
      $hw_type = $3;
    } else {
      $hw_type = "none";
    }

    # Only the CX-700 has 4 buses; the rest of them have only two.

    if ($hw_type eq "700") {
      @buses = (0, 1, 2, 3);
    } else {
      @buses = (0, 1);
    }

    # ktcons has a limit to the size of the _ktcons.ini file, so we have to break them up
    # into smaller files.  We will do one bus for each _ktcons.ini file.

    foreach $bus (@buses) {

      $starting_enclosure = $bus * 8;
      $ktini = "_ktcons_$bus" . ".ini";
      $ktini_path = "$operatingHash{working_dir}\\$ktini";

      # If it's not the first bus, throw in a .source command.  We will throw in
      # a q[uit] command later if this is the last bus.

      # If it's not the first bus, close the previous ktcons ini file.

      if ($bus != $buses[0]) {
	print KTFILE ".source $ktini_path\n";
	close KTFILE;
      }

      open KTFILE, ">$ktini_path" or die "Error: Can't open $ktini_path File: $!\n";

      # Remember the first ktcons ini file that we need to submit to ktcons.
      # Also, emit the !load fdb.dll command.

      if ($bus == $buses[0]) {
	$first_ktcons_ini_path = $ktini_path;
	print KTFILE "!load fdb.dll\n";
      }

      foreach $bus_enclosure (0..7) {
	$enclosure_id = $starting_enclosure + $bus_enclosure;
	$prom_info_file = "prom_info_$enclosure_id" . ".txt";
	$prom_info_path = "$operatingHash{working_dir}\\$prom_info_file";

	open KTCONS_LOG_FILE, ">$prom_info_path" or die "Error: Can't open $prom_info_path File: $!\n";

	if ($enclosure_id != 0) {
	  print KTCONS_LOG_FILE "\n";
	}

	print KTCONS_LOG_FILE "Information for bus $bus enclosure $bus_enclosure\n";
	print KTCONS_LOG_FILE "\n";
	close KTCONS_LOG_FILE;

	$ktcons_log_files [$enclosure_id] = $prom_info_path;

	print KTFILE ".logappend $prom_info_path\n";

	for ($prom_id = 1; $prom_id <= 4; $prom_id++) {
	  print KTFILE "!getprominfo $enclosure_id $prom_id\n";
	}	

	# The CX-700 allows !getprominfo 0 5

	if (0 == $enclosure_id) {
	  if ($hw_type eq "700") {
	    print KTFILE "!getprominfo $enclosure_id 5\n";
	  }
	}

	# If the enclosure id is not 0, then we always allow !getprominfo ID 5
	else {
	  print KTFILE "!getprominfo $enclosure_id 5\n";
	}

	print KTFILE ".logclose\n";
      }
    }
		
    print KTFILE "q\n";
    close KTFILE;

    # Invoke ktcons to do the work.  Can't use &CaptureKtconsLog because I need
    # multiple ktcons input (.ini) files, and for ease of viewing the captured
    # spcollect data, each bus logs to a different ktcons log file (which must
    # be concatenated into one file to then be zipped).  It is easier to have one
    # file to view, but since ktcons cannot write an arbitrary string to the log
    # file (at least, that I know of), we need a header on each and every single
    # log file.
    &InvokeCommand($programHash{"ktcons"}{name}, "-il -f $first_ktcons_ini_path", "");

    # Now, concatenate all the ktcons log files into one bug file.  Can't
    # use &CaptureLogs because I don't want the header or a command to be
    # executed.
    $prom_path = &GenLogName("prom_info.txt");

    &HiliteLogMessage($prom_path, "Information from KTCONS", "PROM info");

    open  PROM_FILE, ">$prom_path" or die "Error: Can't open $prom_path File: $!\n";
    print PROM_FILE "$lineofstars\n";
    print PROM_FILE "PROM Information\n";
    print PROM_FILE "$lineofdashes\n";
    close PROM_FILE;

    foreach $file (@ktcons_log_files) {
      CopyFileInto($file,$prom_path);
      unlink "$file";
    }

    foreach $bus (@buses) {
      $ktini = "_ktcons_$bus" . ".ini";
      $ktini_path = $operatingHash{working_dir} . "\\$ktini";
      unlink "$ktini_path";
    }

    # Now add the prom_info.txt file to the list of file for the sus zip file.

    $ZipIdentifier = "sus";
    &AddCreatedFileToZip($ZipIdentifier, $prom_path);

  }				#end GatherPromInfo


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub :  CaptureKtconsLog
#------------------------------------------------------------------------------------------
# Gather all of the Ktrace data together
#------------------------------------------------------------------------------------------
# bcm - 02/03/06 ktcons now writes to temp file (DIMS138667), copy _ktcons.ini to STDERR
#------------------------------------------------------------------------------------------
sub CaptureKtconsLog
  {
    #Local Variables
    my ($zipIdentifier, $fileName, @commandList) = @_;
    my ($command, $returnVal, $tempFile);
    
    $tempFile = GetTempFilename();

    #Setup all of the necessary parameters to create the log file
    open SCRIPT, ">_ktcons.ini" or print STDERR "Could not open _ktcons.ini for writing: $!\n";
    print SCRIPT "!load fdb\n";
    print SCRIPT ".logappend $tempFile\n";
    foreach $command (@commandList) {
      print SCRIPT "$command\n";
    }
    print SCRIPT ".logclose\n";
    print SCRIPT "q\n";
    close SCRIPT;

    #Print a copy of _ktcons.ini in STDERR
    print STDERR "----======== _ktcons.ini -> $fileName ========----\n";
    open SCRIPT, "<_ktcons.ini" or print STDERR "Could not open _ktcons.ini for reading: $!\n";
    while(<SCRIPT>) {
      print STDERR;
    }
    close SCRIPT;
    print STDERR "----==========================================----\n";
    
    #Create the log file that was just setup
    $returnVal = &InvokeCommand($programHash{"ktcons"}{name}, " -il -f _ktcons.ini", "");
    unlink "_ktcons.ini";

    #Copy results from temp file
    CopyFileInto($tempFile, $fileName);
    
    #Add the log to the list of files to be zipped
    if ($zipIdentifier) {
      &AddCreatedFileToZip($zipIdentifier, $fileName);
    }

    return $returnVal;
  }				#end CaptureKtconsLog

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherRtpFiles
#------------------------------------------------------------------------------------------
# Grab the .out files in c:\temp. (ndu folks ask for these)
#------------------------------------------------------------------------------------------
# wmb - 03/03 - Put all of this code together in its own sub
# dad - 11/04 - Moved the production of all layered app files to ini file
#------------------------------------------------------------------------------------------
sub GatherRtpFiles
  {
    #Local Variables
    my ($file, @tempfiles);

    ###################################
    ## Get the .out files in c:\temp ##
    ###################################

    #Search for the .out files in C:\temp and add them to a list
    my $openTempDirResult = opendir TEMP, "$tempdir"; 
 	if($openTempDirResult){
    
	 	@tempfiles = grep{/ndu-.*\.out/i}readdir TEMP;
	    closedir TEMP;
	
	    #Traverse the list of files appending the complete path to them
	    #Add each file to the appropriate list of files to be zipped
	    foreach $file (@tempfiles) {
	      # Add each of the .out files to the list of files to be zipped
	      SaveFilesToZip("rtp",$tempdir . "\\" . $file);
	    }
	}else{
		&GiveStatus("", "Status: " . $tempdir. " not found. Can't gather RTP files.");
		print STDERR  $tempdir . " not found.\n";
	}
  }				#end GatherRtpFiles

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : ListFiles
#------------------------------------------------------------------------------------------
# Print out all of the .zip files that reside in the dump directory
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Put all of this code together in its own sub
#------------------------------------------------------------------------------------------
sub ListFiles()
  {
    #Local Variables
    my (@zipfiles, $file);

    #Open the directory containing the collects and create a list of all of them
    opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
    @zipfiles = grep{/\.zip$/}readdir DUMP;
    closedir DUMP;

    #Output that the script has now completed building the latest collect
    print "\nEach dump has been collected and zip'ed into its own zip file.\n";
    print "All other data has been collected and zip'ed into a separate zip file.\n";
    print "Gather all of these .zip files and send to EMC-Clariion Support\n";
    print "(These files all live in $dumpdir)\n\n";

    #Output the name of each collect in the directory
    foreach $file (@zipfiles) {
      print "    $file\n\n";
    }

    print "SPCollect has now completed.\nIf you do not see the command prompt please press enter to continue...\n";

  }				#end ListFiles

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GetKtDumpFiles
#------------------------------------------------------------------------------------------
# This function looks for any ktdump files that may be in the dump directory
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Created
#------------------------------------------------------------------------------------------
sub GetKtDumpFiles
  {
    #Local Variables
    my (@ktdfiles, $file);

    #Open the directory containing ktdump files and create a list of all of them
    opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
    @ktdfiles = grep{/ktdump.*\.txt$/i}readdir DUMP;
    closedir DUMP;

    #Check to see if any ktdump files were found
    if (@ktdfiles) {
      # Add each ktdump file to the appropriate list of files to be zipped
      foreach $file (@ktdfiles) {
	SaveFilesToZip("ktd", $dumpdir . "\\" . $file);
      }
    }
  }				#end GetKtDumpFiles

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherFbiFiles
#------------------------------------------------------------------------------------------
# Get the rls output files from fbi
#------------------------------------------------------------------------------------------
#------------------------------------------------------------------------------------------
# kcd - 8/04 - created
#------------------------------------------------------------------------------------------
sub GatherFbiFiles
  {
    my ($filename, @outfiles, $file, @tempfiles);

    ## configuration information file ##
    #Assemble the full name for the config info file
    $filename = &GenLogName( "fbi_info.txt");

    # Put a title line in the file...
	if(!(-e $filename)){
		#adding the log identification string
		&LogMessage($filename, $operatingHash{log_identifier});
	}
	&HiliteLogMessage($filename, "fbi info");

    #add the FBI files to the appropriate list of files to be zipped
    &AddCreatedFileToZip("fbi", $filename);

    #
    # Get the *rls* files in c:\dump
    #

    &GiveStatus("", "Status: Gather c:\\dump\\*rls* files");

    opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
    @tempfiles = readdir DUMP;
    closedir DUMP;

    # for neatness limit the matches to rls* and *.rls, case insensitive

    foreach $file (@tempfiles) {
      if ($file =~ m/(\Arls.*)|(.*\.rls\Z)/i) {
	push @outfiles, $file;
      }
    }

    foreach $file (@outfiles) {
      #add the FBI file to the appropriate list of files to be zipped
      SaveFilesToZip("fbi", $dumpdir . "\\" . $file);
    }
  }				#end GatherFbiInfo


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CopySpaceFile
# usage : CopyFile($group, $zip, $location)
# parameters : $group - The restricted execution groups that this function call belongs to
#              $zip - The nice name of the zip file where the file will be copied to.
#              $path - The path (with spaces in it) to the file to be copied.
#              $filename - The name of the file to be copied
# return values: This function will return $FALSE if the given file could not be copied for
# any reason, or returns $TRUE on a success.
#------------------------------------------------------------------------------------------
# This function will be called by the external configuration file. It will copy a file from
# the given location into the given zip file. Checks are done on the zip file to make sure
# it exists, and checks are done of the path to make sure it exists and to see if it
# contains any environment variable dependent directories.
#
# This function is to be used to copy an existing file that has a pathname that has space(s).
# This function uses a temporary file, without spaces in the pathname, to copy the original
# file into. We also prepend the SP designation to the temporary file (SPx_). This function
# calls the ziphash functionality directly, with a value of "created", since we don't want
# to keep the temporary file.
#------------------------------------------------------------------------------------------
sub CopySpaceFile($$$$)
{
    my ($location, $safefilename);

    #value to be returned
    my $to_return = $TRUE;

    #Setting locals to the passed in parameters
    my($group, $zip, $path, $filename) = @_;

    if(CheckGroup($group)) {

        #Replacing any environment variable names with values in the passed in path
        $path =~ s/%([^%]*)%/$ENV{$1} || "%$1%"/eg;

        # Construct teh full path to the desired file
        $location = "$path"."$filename";

        # Make sure the translation worked
        if (!(-e $location)) {
            $to_return = $FALSE;
        }

        # Now copy to a temp file, with no spaces in the path. Also prepending SPx to the safefilename
        else {
            $safefilename = &GenLogName("$filename");
            CopyFileInto($location, $safefilename);

            #Checking the existence of the file to copy and the zip file
            if (!((-e $safefilename) && ($zipHash{$zip}))) {
                $to_return = $FALSE;
            }

            #Add the file to the list of existing files in the given zip
            else {
                &AddCreatedFileToZip($zip, $safefilename);
            }
        }
    }               # end checkgroup

return $to_return;
}                       #end CopySpaceFile


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CopyFile
# usage : CopyFile($group, $zip, $location)
# parameters : $group - The restricted execution groups that this function call belongs to
#              $zip - The nice name of the zip file where the file will be copied to.
#              $location - The full path of the file to be copied.
# return values: This function will return $FALSE if the given file could not be copied for
# any reason, or returns $TRUE on a success.
#------------------------------------------------------------------------------------------
# This function will be called by the external configuration file. It will copy a file from
# the given location into the given zip file. Checks are done on the zip file to make sure
# it exists, and checks are done of the path to make sure it exists and to see if it
# contains any environment variable dependent directories.
#------------------------------------------------------------------------------------------
sub CopyFile($$$)
  {
    #value to be returned
    my $to_return = $TRUE;

    #Setting locals to the passed in parameters
    my($group, $zip, $location) = @_;

    if(CheckGroup($group)) {
      #Check to see that the proper number of parameters were passed to the function
      if (@_ != 3) {
	$to_return = $FALSE;
      } else {

	#Replacing any environment variable names with values in the passed in path
	$location =~ s/%([^%]*)%/$ENV{$1} || "%$1%"/eg;

	#Checking the existence of the file to copy and the zip file
	if (!((-e $location) && ($zipHash{$zip}))) {
	  $to_return = $FALSE;
	}

	#Add the file to the list of existing files in the given zip
	else {
	  if (!($zipHash{$zip}{$location})) {
            SaveFilesToZip($zip, $location);
	    $to_return = $TRUE;
	  }
	}
      }
    }
    return $to_return;
  }				#end CopyFile

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CopyFiles
# usage : CopyFiles($group, $zip, $directory, $regex)
# parameters : $group - The restricted execution groups that this function call belongs to
#              $zip - The nice name of the zip file where the file will be copied to.
#              $directory - The directory to apply the regular expression to.
#              $regex - Regular expression to match contents of directory with.
# return : 
#------------------------------------------------------------------------------------------
# Contructs an array of file paths for which the regular expression matches against the
# file name and copies these files to the specified zip.
#------------------------------------------------------------------------------------------
sub CopyFiles($$$$)
  {
    my($mode, $zip, $directory, $regEx) = @_;
    my(@tempfiles, $file);
    
    #Replacing any environment variable names with values in the passed in path
    $directory =~ s/%([^%]*)%/$ENV{$1} || "%$1%"/eg;
    
    my $openDirResult = opendir TEMP, $directory;
      if ($openDirResult)
      {
      @tempfiles = grep { /$regEx/ } readdir TEMP;
      closedir TEMP;
      
      foreach $file (@tempfiles)
      {
        $file = "$directory\\$file";
        CopyFile($mode, $zip, $file);
      }
      
      return $TRUE;
    }
    
    return $FALSE;
  }				#end CopyFiles

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CreateFile
# usage : CreateFile($group, $zip, $file_name, $program, @cl_args)
# parameters : $group - The restricted execution groups that this function call belongs to
#              $zip - The nice name of the zip file where the file will be copied to.
#              $file_name - The name of the file to hold the output of the executed program
#              $program - the program to be executed
#              @cl_args - any command line arguments to be passed to the executed program
# return values: This function will return $FALSE if the given file could not be created for
# any reason, or returns $TRUE on a success.
#------------------------------------------------------------------------------------------
# This function will execute the passed in program with the given command line options and 
# then save the output of the program to the passed in file name. This file will then be 
# added to the list of created files for the passed in zip file.
#------------------------------------------------------------------------------------------
sub CreateFile($$$$@)
  {
    #locals
    my ($cl_temp, $cl_string, $counter, $proceed, $message, $label_for_log, $funcReturn);
    my ($invokemode);

    #value to be returned
    my $to_return = $TRUE;

    #Setting locals to the passed in parameters
    my($group, $zip, $file_name, $program, @cl_args) = @_;

    if (CheckGroup($group)) {
      #Check to see that the proper number of parameters were passed to the function
      if (@_ < 4) {
	$to_return = $FALSE;
      } else {

	#Replacing any environment variable names with values in the command line args
	for ($counter = 0; $counter < @cl_args; $counter++) {
	  $cl_args[$counter] =~ s/%([^%]*)%/$ENV{$1} || "%$1%"/eg;
	}

	#See if this program has failed to many times and if so skip its execution
	if ($programHash{$program}{previousStatus} < MAX_PREVIOUSSTATUS) {

          $invokemode = $programHash{$program}{invokemode};
	
	  #check to see if the program should work
	  if (&CanExecute($program)) {
	    #Set value to return to default
	    $funcReturn = $RETURN_BAD;

	    if ($program ne "zip") {
	      #Creating the complete name for the log file
	      $file_name = &GenLogName($file_name);

		  #Add a header to the file being created
		  if(!(-e $file_name)){
		 	#adding the log identification string  
		 	&LogMessage($file_name, $operatingHash{log_identifier}, $lineofstars);     
		    &HiliteLogMessage($file_name, "Information from $program");
		   }
	    }

	    ####
	    #### Each program that can be executed by spcollect has a different set of rules
	    #### to follow. As a result a different path is followed depending on which
	    #### program is called for
	    ####

	    #Ktcons calls have their own subroutine
	    if ($program eq "ktcons") {
	      $funcReturn = &CaptureKtconsLog($zip, $file_name, @cl_args);
	    }

	    #Function calls that MAY rely on layered app information need to be checked
	    elsif ($invokemode eq "UsesLayeredApp") {
	      $funcReturn = &UsesLayeredApp($zip, $file_name, $program, @cl_args);
	    } elsif ($invokemode eq "normal") {

	      foreach $cl_temp (@cl_args) {
                &HiliteLogMessage($file_name, "$program $cl_temp");

                #psmtool has a special invocation case
                if($program eq "psmtool") {
                  $funcReturn = &InvokeCommand($programHash{$program}{name},
                                                        "$cl_temp $file_name", "");

                } else {
                  $funcReturn = &InvokeCommand($programHash{$program}{name},
                  "$cl_temp", $file_name);
                }
	      }
	      if (-e $file_name) {
		&AddCreatedFileToZip($zip, $file_name);
	      }
	    } elsif ($invokemode eq "zip") {
	      $funcReturn = &InvokeCommand($programHash{$program}{name},
                "$file_name ".$cl_args[0], "");
	      if (-e $file_name) {
		if ($zip) {
		  &AddCreatedFileToZip($zip, $file_name);
		} else {
		  $zipHash{$file_name} = "created";
		}
	      }
	    }
	    #The program variable is not one that this script currently knows about
	    else {
	      #adding a note to the runtime log that a bad call was made out of the ini file
	      &GiveStatus("", "Error: A call to the program $program was made out of the ini file.",
			  "The program in question is not known to the spcollect.");
	    }

	    #Setting up return values
	    if ($funcReturn == $RETURN_GOOD) {
	      $to_return = $TRUE;
	    }
	  }

	  #One of the necessary services for the program is not running
	  else {
	    #signal that the program could not run this time
	    $programHash{$program}{previousStatus}++;
	  }
	}

	#The previous_status count is too high which means that the program failed
	#to run multiple times and will now be skipped for the rest of the execution of this program.
	else {
	  #Note that the program will be skipped for the rest of spcollect
	  &GiveStatus("", "The program $program failed to run multiple times.",
		      "Gathering of data from this program is being skipped.");
	}
      }
    }
    return $to_return;
  }				#end CreateFile

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : UsesLayeredApp
# usage : UsesLayeredApp($zip, $file_name, $program, @cl_args)
# parameters : $zip - The nice name of the zip file where the file will be copied to.
#              $file_name - The name of the file to hold the output of the executed program
#              $program - the program to be executed
#              @cl_args - any command line arguments to be passed to the executed program
#------------------------------------------------------------------------------------------
# This function checkes to see if the program being used is navicli, navicimcli, or mvacli 
# (the only three programs part of spcollect that are currently known to use layered app
# information). Checks to see if the log being generated requires the use of a layered app.
# A check is needed to ensure that layered apps being used are enabled or erroneous errors
# may be generated in the SP's ktraces. Next  If all checks are passed then the command is 
# executed and the output is placed in the desired file, and if not then a proper log
#  message is put in place of the command output.
#------------------------------------------------------------------------------------------
sub UsesLayeredApp
  {
    #assigning passed in variables
    my($zip, $file_name, $program, @cl_args) = @_;

    #locals
    my($counter, $message, $returnVal, $cl_temp, @status);

#initialize the return value
$returnVal = $RETURN_BAD;

    #Checks and execution for mvacli
    if ($program eq "mvacli") {

      #Check whether mva is enabled and whether the command can continue
      @status = &CheckLayeredApp($program);

      #Execute the command if mva is enabled
      if ($status[0] == $TRUE) {
	foreach $cl_temp (@cl_args) {
	  &HiliteLogMessage($file_name, "$program $cl_temp");
	  $returnVal = &CaptureLogs("", $file_name, $programHash{$program}{name}, $cl_temp);
	}
	if (-e $file_name) {
	  &AddCreatedFileToZip($zip, $file_name);
	}
      }

      #The layered app being checked is not installed and will be noted in the file
      else {
	print $status[1] . "\n";
	&HiliteLogMessage($file_name, $program);
	&LogMessage($file_name, $status[1]);
	&AddCreatedFileToZip($zip, $file_name);
      }
    }

    #checks and execution for either navicli or navicimom calls
   elsif($program eq "navicli" or "navicimom") {

     ##
     ## Both navicli and navicimom can obtain information from layered apps. 
     ## Erroneous critical errors can be generated by querrying layered apps that are not
     ## installed/enabled. Therefore, we must check to see if the information being 
     ## gathered by each argument requires a layered app and if so whether the app is 
     ## installed/enabled.
     foreach $cl_temp (@cl_args) {
       $returnVal = $RETURN_BAD;

       #check whether argument requires a layered app	
       @status = CheckLayeredApp($cl_temp);

       #if no layered app is needed or if it is enabled the proceed
       if ($status[0] == $TRUE) {
	 $returnVal = &CaptureLogs("", $file_name, $programHash{$program}{name}, $cl_temp);
         #
         # Now add file to list to be added to the appropriate zip file
         #
	 if (-e $file_name) {
	   &AddCreatedFileToZip($zip, $file_name);
	 }
       }

       #the layered app being checked in question is not installed and will be noted in the file
       else {
	 $returnVal = $RETURN_GOOD; #layered app not enabled, but the command didn't fail itself
	 print $status[1] . "\n";
	 &HiliteLogMessage($file_name, $program . " " . $cl_args[$counter]);
	 &LogMessage($file_name, $status[1]);
	 &AddCreatedFileToZip($zip, $file_name);
       }

     }	
   } else {
     $message = "The program being used is not recognized by spcollect!\n"; 
	 print $message;
	 &HiliteLogMessage($file_name, $program);
	 &LogMessage($file_name, $message);
	 &AddCreatedFileToZip($zip, $file_name);
   }

    #
    #  Unfortunately, only the last returnVal is returned and reported...
    #
    return $returnVal;
  }				#end UsesLayeredApp

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CanExecute
# usage : CanExecute($program)
# parameters :
# $program - the program that is going be tested
#------------------------------------------------------------------------------------------
# This function will get a list of all of the services necessary for $program to run by
# looking it up in the program registry and finding all of the services that it depends on.
# Those services will then be checked to make sure that they are all running. The program
# returns true if the correct services are running and returns false otherwise.
# 
# The function also checks that the binary executable is valid.  
#------------------------------------------------------------------------------------------
# bcm - 03/29/06 Added check for zero file size of the executable
#------------------------------------------------------------------------------------------
sub CanExecute($)
  {
    #setting passed in param to local
    my($program) = @_;

    #local variables
    my $can_execute = $TRUE;	#return value of the subroutine
    my ($service, %service_status);

    #Check the passed program against the program registry	
    if ($programHash{$program}) {

      #check to see that each of the necessary services are running
      foreach $service (@{$programHash{$program}{services}}) {
        #Get the current state of a service
        Win32::Service::GetStatus("", $service, \%service_status);

        #Check that the current state is "running"
        #Note: The magic number for "running" is 4, but this is not documented
        if ($service_status{CurrentState} != 4) {
          $can_execute = $FALSE;
        }
      }
    }

    #The program called for is not part of the programHash
    else {
      #note that this program is not part of the programHash
      &GiveStatus("", "\tError: The requested program is not in the programHash");
    }
    return $can_execute;
  }				#end CanExecute

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GatherAllData
#------------------------------------------------------------------------------------------
# This is the master "gather" function.  It is called from main and controls the order
# in which all of the data is gathered.  Any new data that must be collected should be
# its own sub function and be called from this function.
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Created
#------------------------------------------------------------------------------------------
sub GatherAllData
  {
    #locals
    my($file_line, @commands);
    my($path, $dirname);

    #open the external file that contains most function calls
    open(INIFILE, $operatingHash{ini_file_path}) or
      &GiveStatus("", "The ini file could not be opened");

    #Gather all of the information from the file.
    @commands = <INIFILE>;
    close INIFILE;

    #Parse the information and extract the function calls.
    $file_line = join("", @commands);
    $file_line =~ s/\cJ|\cI|\n//g; #replace carriage return/line feed combos with a newline
    #The regex below does the following:
    # Looks for valid command calls in the external file
    # & - look for an ampersand 
    # \w+ - look for some word
    # \( - look for an open parethese
    # (?:)+? - find the regex inside the parentheses at least once, but do not return it
    # .+? - look for at least one non-whitespace characters (minimally)
    # \n* - look for zero or more new lines
    # \); - look for closing paren and then a semicolon
    # g - repeat this until the end of the stream
    # The regex is set to an array so that each successful find of the regex will have its
    # own discrete entry in the array.
    @commands = ($file_line =~ /&\w+\((?:.+?\n*)+?\);/g);

    #execute the function calls one at a time
    foreach $file_line (@commands) {
      if ((eval $file_line) == $FALSE) {
	#note that the line from the ini file was not executed correctly
	&GiveStatus("", "\tError: The line:", "\t$file_line", 
		    "\tWas not executed successfully");
      }
    }

    #Get PROM information
    &GatherPromInfo();

    # Get navi management server files, etc.
    &GetNaviMngrServerData();

    # Get all RTP program's data (ex: snapview, mirrorview)
    &GatherRtpFiles();

    #Get the RLS files from fbi
    &GatherFbiFiles();

    # Get nt event log information
    &GetNtEventLogs();

    # Get ktdump files
    &GetKtDumpFiles();

 
    # Capture navicli getlog
    &GetNaviGetLog();

    #Caputre mini dumps
    &GetMiniDumps(); 

    # Capture dump analysis files
    &GetAnalysisFiles();
    
  
    # Generate c:\\temp\dumpDevMap.out
    # save it to sus
    &InvokeCommand($programHash{"admintool"}{name}, "devmap -dump", "");
    $path = "C:\\temp\\DumpDevMap.out";
    if( -e $path)
     {
      &GiveStatus("", "Status: gathering $path");
      &AddCreatedFileToZip("sus", $path);
     }
    else
     {
      &GiveStatus("", "Status: unable to create C:\\temp\\DumpDevMap.out");
     }
     
    # copy admin_tlddump.txt to sus if it exists
    my $admin_dump_file = "c:\\DUMPS\\admin_tlddump.txt"; 
	if(-e $admin_dump_file){
		&GiveStatus("brief", "Status: copy admin_tlddump.txt");
		&CopyFile("brief", "sus", $admin_dump_file);
	}

	#cleanup the temp file created by AdminTool if it exists.
	#This code can be removed when DIMS 126580 is resolved. 
	my $temp_admin_file = "C:\\tempfmap";
	if (-e $temp_admin_file){
		unlink $temp_admin_file; 
	}
	
	     
   $dirname = GenLogName("dir.txt");
   &InvokeCommand_("dir", "c:\\ /s /os", $dirname);
   &AddCreatedFileToZip("sus", $dirname);     
  }				#end GatherAllData

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CaptureNorthstarLogs
#------------------------------------------------------------------------------------------
# This code searches c:\dump for all files that match m/\S*_drive_log_\d{2}_\d{2}_\S*\.txt/
# All matched files are added to the sus zip and deleted
#------------------------------------------------------------------------------------------
sub CaptureNorthstarLogs($$)
  {
    my ($group, $zip) = @_;
    my ($file_name, $path, @files);
    &GiveStatus("", "Status: in CaptureLogs");

    if (CheckGroup($group)) {
      opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
      @files = readdir DUMP;
      closedir DUMP;

      #add the files to the list of files to be zipped and deleted
      foreach $file_name (@files)
      { 
       if ($file_name =~ m/^\S*_drive_log_\d{2}_\d{2}_\S*\.txt$/)
         {
          $path = "$dumpdir\\$file_name";
          &GiveStatus("", "Status: gathering $path");
	  &AddCreatedFileToZip($zip, $path);
	 }
      }
   }
   return $TRUE;
  }


#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CaptureLccStats
#------------------------------------------------------------------------------------------
# This code searches c:\dump for all files that match serialnumber_sp(a/b)_lccstats.log.
# All matched files are added to the lcc zip and deleted
#------------------------------------------------------------------------------------------
sub CaptureLccStats($$)
  {
    my ($group, $zip) = @_;
    my ($file_name, $path, @files);
    
    if (CheckGroup($group)) {
      opendir DUMP, "$dumpdir" or die "Unable to open dump directory!";
      @files = grep /^.*_sp[AB]_lccstats\.log$/i, readdir DUMP;
      closedir DUMP;

      #add the files to the list of files to be zipped and deleted
      foreach $file_name (@files)
      { 
        $path = "$dumpdir\\$file_name";
        &GiveStatus("", "Status: gathering $path");
	&AddCreatedFileToZip($zip, $path);
      }
   }
   return $TRUE;
  }             # End of CaptureLccStats($$)

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CaptureLccLogs
#------------------------------------------------------------------------------------------
# This code searches c:\dump for all files that match m/SP[ABab].*yukon_log_encl.*\.xml
# All matched files are added to the lcc zip and deleted
#------------------------------------------------------------------------------------------
sub CaptureLccLogs($$)
  {
    my ($group, $zip) = @_;
    my ($path);


    &GiveStatus("", "Status: in CaptureLccLogs");

    if (CheckGroup($group))
    {
      if (CanExecute("spcyukoncollect"))
      {
        # Call spcyukoncollect utility
        $path = "$operatingHash{working_dir}\\$zipHash{lcc}{name}";

        InvokeCommand("$programHash{spcyukoncollect}{name}",$path,"");

        #Zip created by utility, put in $zipHash
        &AddCreatedFileToZip("lcc", $path);
      }
    }

    return $TRUE;
  }


sub CreateCopyOfFile
 {
  my($src, $dest) = @_;
  my($line);

  my $r1 = open(SRC, $src);
  if(!$r1){
  	print STDERR "$src" . " not found. \n";
  	return;  
  }

  my $r2 = open(DST, ">$dest");
  if(!$r2){
  	print STDERR "$dest" . " not found. \n";
  	close(SRC);
  	return;    
  }
  
  while($line = <SRC>)
   { print DST $line; }
  close(SRC);
  close(DST);
 }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CreateCollection
#------------------------------------------------------------------------------------------
# This is the master "compress" function.  It zips up all of the files into one big
# "glob" zip.  If you are debuging/testing, you may want to remove the "-m" switch.
#------------------------------------------------------------------------------------------
# wmb - 3/03 - Created
#------------------------------------------------------------------------------------------
sub CreateCollection
  {
    #Local Variables
    my($list_created, $list_copied, $collection, $dummyOut, $file_key, $value, $inv_file, @inv_list);

    #create a filename for, label, and open the inventory file
    $inv_file = &GenUniqueLogName("zip_inventory.txt");
    &HiliteLogMessage($inv_file, "Inventory of " . $programHash{zip}{name});

    #add runtime logs, rev file, and inventory file
    #to the list of file being zipped into the
    #and add them to the inventory list
    $zipHash{$operatingHash{stdo_file}} = "created";
    $zipHash{$operatingHash{stde_file}} = "created";
    $zipHash{$operatingHash{log_file}} = "created";
    $zipHash{$operatingHash{fname_name_rev}} = "created";
    $zipHash{$inv_file} = "created";

    $operatingHash{spcollect_log_file} = &GenLogName("SPCollect_log.txt");    
    CreateCopyOfFile("C:\\dumps\\SPCollect_log.txt", $operatingHash{spcollect_log_file});
    $zipHash{$operatingHash{spcollect_log_file}} = "created";
     
    $operatingHash{peerwatch_log_file} = &GenLogName("PeerWatchLog.txt");
    CreateCopyOfFile("C:\\dumps\\PeerWatchLog.txt", $operatingHash{peerwatch_log_file});
    $zipHash{$operatingHash{peerwatch_log_file}} = "created";

    &SaveFilesToZip("", $operatingHash{ini_file_path},$operatingHash{spcollect_path});
    &LogMessage($inv_file, "\n" . $programHash{zip}{name},
		"\t" . $operatingHash{stdo_file},
		"\t" . $operatingHash{stde_file},
	        "\t" . $operatingHash{log_file}, 
	        "\t" . $operatingHash{fname_name_rev}, 
	        "\t" . $operatingHash{spcollect_log_file}, 
	        "\t" . $operatingHash{peerwatch_log_file},
	        "\t" . $inv_file);

    ##########################################################################
    # Traverse through the zip hash from the bottom up. On each level add the
    # files listed to their respective zip files and add the names of the
    # files to the inventory list file.
    # Finally package all zips into the main SPCollect zip file
    ##########################################################################
    foreach $collection (keys %zipHash) {
      if(ref($zipHash{$collection}) eq "HASH")
	{
	  #Reset the strings of created and copied files
	  $list_created = "";
	  $list_copied = "";

	  #Add key to the inventory file
	  &LogMessage($inv_file, "\t" . $zipHash{$collection}{name});

	  #Construct a string of files that were created and copied for zipping
	  #Add each file to be added to the zip to the inventory file
	  foreach $file_key (keys %{$zipHash{$collection}}) {

	    if ($zipHash{$collection}{$file_key} eq "created") {
	      $list_created = $list_created . " " . $file_key;
	      $file_key =~ /((?:\w*\.*-*)+)$/;
	      &LogMessage($inv_file, "\t\t" . $1);
	    } elsif ($zipHash{$collection}{$file_key} eq "copied") {
	      $list_copied = $list_copied . " " . $file_key;
	      $file_key =~ /((?:\w*\.*-*)+)$/;
	      &LogMessage($inv_file, "\t\t" . $1);
	    }

	  }

	  #zip the created files and delete the originals
	  if ($list_created) {
	    &CreateFile("always", "", $zipHash{$collection}{name}, "zip", " -m -j -g -q  $list_created");
	  }

	  #zip the copied files but leave the originals
	  if ($list_copied) {
	    &CreateFile("always", "", $zipHash{$collection}{name}, "zip", " -j -g -q  $list_copied");
	  }

	  #if anything was added to it, add this file to the list going into the final zip
	  if (-e $zipHash{$collection}{name}) {
            #&GiveStatus("", "zipHash: $zipHash{$collection}{name} = created/");
	    $zipHash{$zipHash{$collection}{name}} = "created";
	  }
	}
    }

    #restart the fbi service
    &InvokeCommand($programHash{"monitorfbi"}{name}, "-resume", "");


    # Lastly: Add ending timestamp to the collection rev file
    &LogMessage($operatingHash{fname_name_rev}, "End time of operation:" . &FilenameDT());

    #Reset the string of files created/copied
    $list_created = "";
    $list_copied = "";

    #Roll all of the generated files into the main zip
    #if there are generated files then zip them together and delete the originals
    foreach $file_key (keys %zipHash) {
      $value = $zipHash{$file_key};
      if ($value eq "created") {
	#set the string of files to be zipped to a single string
	$list_created = $list_created . " " . $file_key;
        #&GiveStatus("", "list_created: Storing $file_key");
      }
      elsif ($value eq "copied") {
        $list_copied = $list_copied . " " . $file_key;
        #&GiveStatus("", "list_copied: Storing $file_key");
      }
    }


    #Restore the STDERR stream
    &RestoreStd();

    #Create the final Zip file
    # First add all existing files, which can not be deleted
    if($list_copied ne "") {
      #&GiveStatus("", "list_copied: Storing $list_copied");
      $dummyOut = $programHash{zip}{name} . " -j -q -g " . $dumpdir . "\\" .
        $zipHash{name} . " " . $list_copied;
      $dummyOut = `$dummyOut`;
    }

    # Second, add and delete all generated files
    if($list_created ne "") {
      #&GiveStatus("", "list_created: Storing $list_created");
      $dummyOut = $programHash{zip}{name} . " -m -j -q -g " . $dumpdir . "\\" .
        $zipHash{name} . " " . $list_created;
      $dummyOut = `$dummyOut`;
    }


    #The collect file has been created, so delete the working directory
    chdir($dumpdir);
    rmtree($operatingHash{working_dir});
	
  }				#end CreateCollection

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : RunLogIt
#------------------------------------------------------------------------------------------
# This function accepts a string to log to the log file.  It will add a time stamp to
# the entry also.
# Arguments:
# The first argument should be the string to log.
#------------------------------------------------------------------------------------------
# wmb - 5/19 - Created
#------------------------------------------------------------------------------------------
sub RunLogIt
  {
    #Local Variables
    my (%timehash, $cur_time, $log_string);

    #Get the current time and date into a local data structure
    %timehash = GetGMTime();

    # adjust other single digits to have a leading zero
    FormatTimeHash(\%timehash);

    #Format the time as desired
    $cur_time = "[$timehash{hour}:$timehash{min}:$timehash{sec}]";

    #Add the passed in message to the timestamp
    $log_string = "$cur_time $_[0]\n";

    #Open a handle to the runtime log
    open LOG_FILE_H, ">>" . $operatingHash{log_file}
      or die "Could not open run log file" . $operatingHash{log_file} . "\n";

    #Add the message to the console
    print $log_string;

    #Add the message to runtime log
    print LOG_FILE_H $log_string;

    close LOG_FILE_H;

  }				#end RunLogIt

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : LogMessage
#------------------------------------------------------------------------------------------
# This function is used to add a messages to a log file
# Arguments:
# The first argument is the file the message should be logged to
# The remaining arguments are appended to the log file, an lf is added after argument
#------------------------------------------------------------------------------------------
# mjb - 03/09/29 -   created
#------------------------------------------------------------------------------------------
#
sub LogMessage
  {
    #Assigning passed in parameters
    my ($logFile, @strings) = @_;

    #Local Variables
    my ($string);
	
    #Open the desired file and add the passed in string to it.
    open(LOGFILE, ">>$logFile") || die "problem opening $logFile";
    foreach $string (@strings) {
      print LOGFILE "$string\n"; 
    }
    close(LOGFILE);

  }				#end LogMessage

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : HiliteLogMessage
#------------------------------------------------------------------------------------------
# This function is used to add a command messages to a log file
# The command message is preceeded by a line of stars
# and followed by line of dashes
#
# Arguments:
# The first argument is the file the message should be logged to
# The second argument is the command
#------------------------------------------------------------------------------------------
# mjb - 03/09/29 -   created
#------------------------------------------------------------------------------------------
sub HiliteLogMessage
  {
    #Assigning passed in parameters
    my ($logFile, @strings) = @_;

    #Log the message with stars preceding and dashes following it
    &LogMessage($logFile, $lineofstars, @strings, $lineofdashes);
  }				#end HiliteLogMessage

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : SaveFilesToZip
#------------------------------------------------------------------------------------------
# This function adds existing files to the appropriate list of files to be zipped
#------------------------------------------------------------------------------------------
sub SaveFilesToZip()
  {
    #Assigning passed in parameters
    my ($zipIdentifier,  @files) = @_;

    #Local Variables
    my ($afile, $file_key, $value);

    #Add each existing file in the list to the appropriate list of files to be zipped
    foreach $afile (@files) {
      if ( -e $afile) {
        if($zipIdentifier eq "") {
	  # an empty zipIdentifier identifies the primary zipHash/data.zip
	  $zipHash{$afile} = "copied";
        }
        else {
	  # The file needs to be copied to a particular zip file
	  $zipHash{$zipIdentifier}{$afile} = "copied";
        }
        #&GiveStatus("", "Saving $afile to $zipIdentifier");
      }
      else {
        &GiveStatus("", "SaveFilesToZip: missing $afile");
      }
    
    }
  }				#end SaveFilesToZip

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CaptureLogs
#------------------------------------------------------------------------------------------
# This function invokes a set of commands and gathers the output into log.
# That log is then added (if necessary) to the appropriate list of files to be zipped
#------------------------------------------------------------------------------------------
sub CaptureLogs()
  {
    #Assigning passed in parameters
    my ($zipIdentifier, $fileName, $program, $args) = @_;

    #Local Variables
    my ($returnVal);

    #Log each of the commands that will be run
    #Then run the commmand
    &HiliteLogMessage($fileName, "$program $args");
    $returnVal = &InvokeCommand($program, $args, $fileName);

    #Add one more line of dashes to the log file
    &LogMessage($fileName, $lineofdashes);

    #Add the log to the appropriate list of files to be zipped
    if ($zipIdentifier) {
      $zipHash{$zipIdentifier}{$fileName} = "created";
    }
    return $returnVal;
  }				#end CaptureLogs()

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : CheckValidBinary
# usage : CheckValidBinary($file)
# parameters : $file  - File to check
# return: $TRUE  - File is a valid executable
#         $FALSE - File is not a valid executable
#------------------------------------------------------------------------------------------
# This function checks that a binary executable file is valid.
# NOTE
# Currently a file size of zero is checked for, but non-zero sized executables still need
# to be verified to prevent a possible process hang bug.  Possible solution may include
# some sort of hash checking (ie MD5 checksum).
#------------------------------------------------------------------------------------------
# bcm - 03/28/06 - Created
#------------------------------------------------------------------------------------------
sub CheckValidBinary($)
  {
    my ($file) = @_;
    if (-z $file || not(-e $file)) {
      return $FALSE;
    }
    return $TRUE;
  }                             #end CheckValidBinary()

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : InvokeCommand
# usage : InvokeCommand($command_string, $log_file)
# parameters : $command      - command to execute
#              $command_args - command arguments
#              $log_file     - log file to append
#------------------------------------------------------------------------------------------
# This function is used to run a system command.  It should be used instead of directly
# invoking a system command so that all commands can be logged to the run log and 
# status can be checked.
#------------------------------------------------------------------------------------------
# wmb - 5/19 - Created
# bcm - 01/31/06 - Use system, redirect STDOUT to temp file, copy tempfile to log file
# bcm - 03/28/06 - Added zero file size check to prevent hung processes from an invisible
#                  Win32 dialog when a executable is not valid, non-zero but "valid" binary
#                  files are still a possible problem.
# bcm - 03/29/06 - Arguments passed in seperate from executable
#------------------------------------------------------------------------------------------
sub InvokeCommand($$$)
  {
    #Assigning passed in parameters
    my($command, $command_args, $log_file) = @_;





    #Handle zero sized executable error
    if (CheckValidBinary($command) eq $FALSE)
    {
      &RunLogIt("Error: Executable file size is 0 ($command)");
      return -1;
    };
   return InvokeCommand_($command, $command_args, $log_file);
  }				#end InvokeCommand

sub InvokeCommand_($$$)
  {
    #Assigning passed in parameters
    my($command, $command_args, $log_file) = @_;

    #Local Variables
    my($system_return, $log_string, $tempFile, $command_string);

    $command_string = $command." ".$command_args;

    #Assemble a message stating the command to be run
    $log_string = "Invoking Command: [$command_string]";

    #Add the command message to the runtime log
    &RunLogIt($log_string);

    # Redirect STDOUT to temp file
    $tempFile = GetTempFilename();

    RedirectSTDOUT($tempFile);

    #Run the actual command and record the output (mostly at this point to return)
    system $command_string;

    # Restore STDOUT
    RestoreSTDOUT();

    # Copy temp file to log_file
    if ( not($log_file eq "") ) {
      CopyFileInto($tempFile, $log_file);
    }

    #If the status of the command just run is not 0,
    #then add the status returned to the runtime log.
    if ($? != $RETURN_GOOD) {
      $log_string = "Warning: Return Value was: $? (Return code ".($? >> 8).")\n";
      &RunLogIt($log_string);
    }

    #Return the status of the command that was run
    return $?;
  }

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GenLogName
#------------------------------------------------------------------------------------------
# This function generates an sp specific log name from a key
#  Argument 1 is the key.
#------------------------------------------------------------------------------------------
sub GenLogName
  {
    return $operatingHash{sp_id} . "_" . $_[0];
  }				#GenLogName

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# sub : GenUniqueLogName
#------------------------------------------------------------------------------------------
# This function generates an sp specific and unique log name from a key
#  Argument 1 is the key.
#------------------------------------------------------------------------------------------
sub GenUniqueLogName
  {
    return $operatingHash{sp_serial}  . "_" . $operatingHash{sp_id} . "_" . 
           $operatingHash{file_date}.  "_" . $_[0];
  }				#end GenUniqueLogName

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : LabelLog
#------------------------------------------------------------------------------------------
# Usage: &LabelLog($group, $file, @notes)
# Purpose: This function is called by the ini file to put a not in a file that is being
# generated. The function will create the file if this is to be the first entry into it.
# Parameters: $group - The restricted execution groups that this function call belongs to
#             $file - The name of the file to be created or appended to
#             @notes - An array of strings to be appended to the file
#------------------------------------------------------------------------------------------
# dad - 11/04 - Created
#------------------------------------------------------------------------------------------
sub LabelLog($$@)
  {
    #local variables
    my($group, $file, @notes) = @_;

    if (CheckGroup($group)) {
      #create the fully qualified name for the file
      $file = &GenLogName($file);

      if (!(-e $file)) {
	#adding the log identification string
	 &LogMessage($file, $operatingHash{log_identifier}, $lineofstars); 
      }

      #Call the function responsible for the logging
      &HiliteLogMessage($file, @notes);
    }
  }				#end LabelLog

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : GiveStatus
#------------------------------------------------------------------------------------------
# Usage: &GiveStatus($group, @status_strings);
# Purpose: This function will send the passed in strings to the stdout log file, as well as
#          to STDOUT itself. This function is used whenever status about what spcollect is 
#          doing must be given.
# Parameters: $group - The restricted execution groups that this function call belongs to
#             @status_strings - The messages to send to stdout log
#------------------------------------------------------------------------------------------
# dad - 11/04 - created
#------------------------------------------------------------------------------------------
sub GiveStatus($@)
  {
    #local variables
    my($temp_msg);

    #Setting passed in vars
    my($group, @status_strings) = @_;

    if (CheckGroup($group)) {
      #Opening the stdout log file for appending
      open STDOUT_FILE, ">>" . $operatingHash{stdo_file}
	or die "Could not open stdout file" . $operatingHash{stdo_file} . "\n";

      #posting each of the messages and then adding one extra newline for neatness
      foreach $temp_msg (@status_strings) {
	print STDOUT_FILE "$temp_msg\n";
	print STDOUT "$temp_msg\n";
      }
      print STDOUT_FILE "\n";
      print STDOUT "\n";
      close STDOUT_FILE;
    }
    return $TRUE;
  }				#end GiveStatus

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CheckGroup
#------------------------------------------------------------------------------------------
# Usage: &CheckGroup($group);
# Purpose: This function is designed to filter out function calls when a reduced spcollect
# is called for. A reduced spcollect is specified by command line flags, and then those
# flags are compared for each function call to the group memberships listed in the $group 
# variable. The function returns $TRUE if this function call is supposed to run now, and
# returns false if it not supposed to run now.
# Parameters: $group - The restricted execution groups that this function call belongs to
#------------------------------------------------------------------------------------------
# dad - 12/04 - created
#------------------------------------------------------------------------------------------
sub CheckGroup($) {
  #setting passed in vars
  my($group) = @_;

  #local variabls
  my($to_return, $argument);

  # override, which ensures that zip files will always be created
  if($group eq "always")
   { return $TRUE; }


  #Determine whether this function call is supposed to run now by checking to see if:
  #There were no restrictive arguments passed to spcollect or
  #That $groups contains at least one of the arguments passed to spcollect
  if( !defined $argHash{'mode'} || $group =~ m/$argHash{'mode'}/ ) {
    $to_return = $TRUE;
  } else {
    $to_return = $FALSE;
  }

  return $to_return;
}	#end CheckGroup	

#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# Sub : CheckLayeredApp
#------------------------------------------------------------------------------------------
# Usage: &CheckGroup($program);
# Purpose: This function checks whether layered app information is supposed to be gathered
# for a given command, and if so checks to see if the layered app is enabled. This function
# returns an array with two members. The entries are as follows:
#
# The first entry in the array returns $TRUE if a layered app is called for and is enabled 
# or if a layered app is not called for at all. It returns $FALSE if a layered app is 
# called for and is not enabled.
#
# The second entry in the array is a message string to be used for logging.
#
# Parameters: $program - This is the value to check to see if it calls for a layered app to
# be enabled. Currently this value can be one of three things:
#     -The first argument passed to a Navicli call (which may require a layered app)
#     -The first argument passed to a Navicimom call (which may require a layered app)
#     -"mvacli" for a call to mvacli (which obviously requires mva to be enabled)
#------------------------------------------------------------------------------------------
# dad - 01/05 - created
#------------------------------------------------------------------------------------------
sub CheckLayeredApp($) {
  #setting passed in vars
  my($program) = @_;

  #create local vars
  my ($proceed, $message, $package, $key);
  ($package, $key) = split(/ /, $program);
  $proceed = $FALSE;
  $message = "Can't run  $program \n $package is NOT an enabled package";

  if ($program =~ /mirrorview|mirror/i) {
    if( $key =~ /-async/i) {
      $message = "Can't run  $program \n $package -async is NOT an enabled package";
      if ($ENV{'MVAOPTION_REVISION'}) {
        $proceed = $TRUE;
        $message = "Can run $program \n $package -async is an enabled package";
      }
    }
    else {
      if ($ENV{'MIRRORVIEWOPTION_REVISION'}) {
        $proceed = $TRUE;
        $message = "Can run $program \n $package is an enabled package";
      }
    }
  } elsif ($program =~ /sancopy/i) {
    if ($ENV{'SANCOPYOPTION_REVISION'}) {
      $proceed = $TRUE;
      $message = "Can run $program \n $package is an enabled package";
    }
  } elsif ($program =~ /snapview|clone/i) {
    if ($ENV{'SNAPVIEWOPTION_REVISION'}) {
      $proceed = $TRUE;
      $message = "Can run $program \n $package is an enabled package";
    }
  } elsif ($program =~ /mvacli/i) {
    if ($ENV{'MVAOPTION_REVISION'}) {
      $proceed = $TRUE;
      $message = "Can run $program \n $package is an enabled package";
    }
  } else {
    #we assume that the command is not requesting information from a layered app
    $proceed = $TRUE;
    $message = "The Layered app $program is enabled, Proceed with the function call.";
  }
  return ($proceed, $message);

    } #end CheckLayerdApp
###########################################################################################
##########                             End Subroutines                           ##########
###########################################################################################

###########################################################################################
##########                               main                                    ##########
###########################################################################################

#print the revision of spcollect
print "\nspcollect.pl revision $revision\n";

#Do some setup and checking before we start the real work
&PreRunSetup(); 

#Gather all data from the array that we want to send back
&GatherAllData();

# Collect all files into the master collection zip file
&CreateCollection();

# Final activity: list to stdout all zip files that live in c:\dumps
&ListFiles();

exit;

###########################################################################################
##########                             End Program                               ##########
###########################################################################################
